package org.implab.gradle.containers;

import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.artifacts.Dependency;
import org.gradle.api.file.ProjectLayout;
import org.gradle.api.model.ObjectFactory;
import org.gradle.api.plugins.ExtraPropertiesExtension;
import org.gradle.api.tasks.Copy;
import org.gradle.api.tasks.Delete;
import org.gradle.api.tasks.TaskProvider;
import org.implab.gradle.containers.tasks.BuildImage;
import org.implab.gradle.containers.tasks.PushImage;
import org.implab.gradle.containers.tasks.RunImage;
import org.implab.gradle.containers.tasks.SaveImage;
import org.implab.gradle.containers.tasks.TagImage;

public class ContainerPlugin implements Plugin<Project> {

    public static final String BUILD_GROUP = "build";

    public static final String CONTAINER_EXTENSION_NAME = "container";

    public static final String ARCHIVE_CONFIGURATION = "archive";

    private ContainerExtension containerExtension;

    void exportClasses(Project project, Class<?>... classes) {
        ExtraPropertiesExtension extras = project.getExtensions().getExtraProperties();
        for (var clazz : classes)
            extras.set(clazz.getSimpleName(), clazz);
    }

    public void apply(Project project) {
        ObjectFactory factory = project.getObjects();
        ProjectLayout layout = project.getLayout();
        containerExtension = new ContainerExtension(factory, layout, project);

        containerExtension.getImageAuthority()
                .convention(project.provider(() -> (String) project.getProperties().get("imagesAuthority")));

        containerExtension.getImageGroup()
                .convention(project.provider(() -> (String) project.getProperties().get("imagesGroup")));

        project.getExtensions().add(CONTAINER_EXTENSION_NAME, containerExtension);

        exportClasses(
                project,
                BuildImage.class, PushImage.class, SaveImage.class, TagImage.class, RunImage.class);

        project.getConfigurations().create(Dependency.DEFAULT_CONFIGURATION, c -> {
            c.setCanBeConsumed(true);
            c.setCanBeResolved(false);
        });

        project.getConfigurations().create(ARCHIVE_CONFIGURATION, c -> {
            c.setCanBeConsumed(true);
            c.setCanBeResolved(false);
        });

        TaskProvider<Copy> processResourcesTask = project.getTasks().register("processResources", Copy.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.from(layout.getProjectDirectory().dir("src/main"));
            t.into(containerExtension.getContextDirectory());
        });

        TaskProvider<BuildImage> buildImageTask = project.getTasks().register("buildImage", BuildImage.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(processResourcesTask);

            t.getContextDirectory().set(containerExtension.getContextDirectory());
            t.getImageIdFile().set(containerExtension.getImageIdFile());

            t.getImageName().set(containerExtension.getImageName());
        });

        project.getTasks().register("clean", Delete.class, t -> {
            t.delete(layout.getBuildDirectory());
        });

        project.getTasks().register("build", t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(buildImageTask);
        });

        project.getTasks().register("pushImage", PushImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getImageName().set(buildImageTask.flatMap(b -> b.getImageName()));
        });

        project.getTasks().register("saveImage", SaveImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getImage().set(buildImageTask.flatMap(b -> b.getImageName()));
        });

        project.getArtifacts().add(Dependency.DEFAULT_CONFIGURATION, buildImageTask.flatMap(x -> x.getImageIdFile()),
                t -> {
                    t.builtBy(buildImageTask);
                });
    }
}
